searchData={"items":[{"type":"module","title":"Membrane.AudioMixer","doc":"This element performs audio mixing.\n\nAudio format can be set as an element option or received through stream_format from input pads. All\nreceived stream_format have to be identical and match ones in element option (if that option is\ndifferent from `nil`).\n\nInput pads can have offset - it tells how much silence should be added before first sample\nfrom that pad. Offset has to be positive.\n\nMixer mixes only raw audio (PCM), so some parser may be needed to precede it in pipeline.","ref":"Membrane.AudioMixer.html"},{"type":"module","title":"Element options - Membrane.AudioMixer","doc":"Passed via struct `t:Membrane.AudioMixer.t/0`\n\n- `stream_format`  \n\n  ```\n  RawAudio.t()\n  ```\n  \n  Default value: `nil`  \n  The value defines a raw audio format of pads connected to the\n  element. It should be the same for all the pads.\n\n\n- `frames_per_buffer`  \n\n  ```\n  pos_integer()\n  ```\n  \n  Default value: `2048`  \n  Assumed number of raw audio frames in each buffer.\n  Used when converting demand from buffers into bytes.\n\n\n\n- `prevent_clipping`  \n\n  ```\n  boolean()\n  ```\n  \n  Default value: `true`  \n  Defines how the mixer should act in the case when an overflow happens.\n  - If true, the wave will be scaled down, so a peak will become the maximal\n  value of the sample in the format. See `Membrane.AudioMixer.ClipPreventingAdder`.\n  - If false, overflow will be clipped to the maximal value of the sample in\n  the format. See `Membrane.AudioMixer.Adder`.\n\n\n\n- `native_mixer`  \n\n  ```\n  boolean()\n  ```\n  \n  Default value: `false`  \n  The value determines if mixer should use NIFs for mixing audio. Only\n  clip preventing version of native mixer is available.\n  See `Membrane.AudioMixer.NativeAdder`.\n\n\n\n- `synchronize_buffers?`  \n\n  ```\n  boolean()\n  ```\n  \n  Default value: `false`  \n  The value determines if mixer should synchronize buffers based on pts values.\n  - If true, mixer will synchronize buffers based on its pts values. If buffer pts value is lower then the current\n  mixing time (last_ts_sent) it will be dropped.\n  - If false, mixer will take all incoming buffers no matter what pts they have and put it in the queue.","ref":"Membrane.AudioMixer.html#module-element-options"},{"type":"module","title":"Pads - Membrane.AudioMixer","doc":"### `:input`\n\nAccepted formats:\n```\n%RawAudio{sample_format: sample_format}\nwhen sample_format in [:s8, :s16le, :s16be, :s24le, :s24be, :s32le, :s32be]\n```\n```\nRemoteStream\n```\n\nDirection: | `:input`\nAvailability: | `:on_request`\nFlow control: | `:manual`\nDemand unit: | `:bytes`\nPad options:\n\n- `offset`  \n\n  ```\n  Time.non_neg()\n  ```\n  \n  Default value: `0`  \n  Offset of the input audio at the pad.\n\n\n\n### `:output`\n\nAccepted formats:\n```\nRawAudio\n```\n\nDirection: | `:output`\nAvailability: | `:always`\nFlow control: | `:manual`\nDemand unit: | `nil`","ref":"Membrane.AudioMixer.html#module-pads"},{"type":"function","title":"Membrane.AudioMixer.options/0","doc":"Returns description of options available for this module","ref":"Membrane.AudioMixer.html#options/0"},{"type":"type","title":"Membrane.AudioMixer.input_pad_opts/0","doc":"Options for pad `:input`","ref":"Membrane.AudioMixer.html#t:input_pad_opts/0"},{"type":"type","title":"Membrane.AudioMixer.t/0","doc":"Struct containing options for `Membrane.AudioMixer`","ref":"Membrane.AudioMixer.html#t:t/0"},{"type":"module","title":"Membrane.AudioMixerBin","doc":"Bin element distributing a mixing job between multiple `Membrane.AudioMixer` elements.\n\nA tree of AudioMixers is created according to `max_inputs_per_node` parameter:\n- if number of input tracks is smaller than `max_inputs_per_node`, only one AudioMixer element is created for the entire job\n- if there are more input tracks than `max_inputs_per_node`, there are created enough mixers so that each mixer has at most\n`max_inputs_per_node` inputs - outputs from those mixers are then mixed again following the same rules -\nanother level of mixers is created having enough mixers so that each mixer on this level has at most\n`max_inputs_per_node` inputs (those are now the outputs of the previous level mixers).\nLevels are created until only one mixer in the level is needed - output from this mixer is the final mixed track.\n\nBin allows for specifying options for `Membrane.AudioMixer`, which are applied for all AudioMixers.\n\nRecommended to use in case of mixing jobs with many inputs.\n\nA number of inputs to the bin must be specified in the `number_of_inputs` option.","ref":"Membrane.AudioMixerBin.html"},{"type":"module","title":"Bin options - Membrane.AudioMixerBin","doc":"Passed via struct `t:Membrane.AudioMixerBin.t/0`\n\n- `max_inputs_per_node`  \n\n  ```\n  pos_integer()\n  ```\n  \n  Default value: `10`  \n  The maximum number of inputs to a single mixer in the mixers tree. Must be at least 2.\n\n\n- `mixer_options`  \n\n  ```\n  AudioMixer.t()\n  ```\n  \n  Default value: `%Membrane.AudioMixer{stream_format: nil, frames_per_buffer: 2048, prevent_clipping: true, native_mixer: false, synchronize_buffers?: false}`  \n  The options that would be passed to each created AudioMixer.\n\n\n\n- `number_of_inputs`  \n\n  ```\n  pos_integer()\n  ```\n  \n  ***Required***  \n  The exact number of inputs to the bin. Must be at least 1.","ref":"Membrane.AudioMixerBin.html#module-bin-options"},{"type":"module","title":"Pads - Membrane.AudioMixerBin","doc":"### `:input`\n\nAccepted formats:\n```\n%RawAudio{sample_format: sample_format}\nwhen sample_format in [:s8, :s16le, :s16be, :s24le, :s24be, :s32le, :s32be]\n```\n```\nMembrane.RemoteStream\n```\n\nDirection: | `:input`\nAvailability: | `:on_request`\nPad options:\n\n- `offset`  \n\n  ```\n  Time.t()\n  ```\n  \n  Default value: `0`  \n  Offset of the input audio at the pad.\n\n\n\n### `:output`\n\nAccepted formats:\n```\nRawAudio\n```\n\nDirection: | `:output`\nAvailability: | `:always`","ref":"Membrane.AudioMixerBin.html#module-pads"},{"type":"function","title":"Membrane.AudioMixerBin.gen_mixing_spec/3","doc":"Generates a spec for a single mixer or a tree of mixers.\n\nLevels of the tree will be 0-indexed with tree root being level 0\nFor a bottom level of mixing tree (leaves of the tree) links generator will be used to generate links between inputs and mixers.","ref":"Membrane.AudioMixerBin.html#gen_mixing_spec/3"},{"type":"function","title":"Membrane.AudioMixerBin.options/0","doc":"Returns description of options available for this module","ref":"Membrane.AudioMixerBin.html#options/0"},{"type":"type","title":"Membrane.AudioMixerBin.input_pad_opts/0","doc":"Options for pad `:input`","ref":"Membrane.AudioMixerBin.html#t:input_pad_opts/0"},{"type":"type","title":"Membrane.AudioMixerBin.t/0","doc":"Struct containing options for `Membrane.AudioMixerBin`","ref":"Membrane.AudioMixerBin.html#t:t/0"},{"type":"module","title":"Membrane.LiveAudioMixer","doc":"This element performs audio mixing for live streams.\n\nLive Audio Mixer starts to mix audio after the first input pad is added or, if `latency` option is set to `nil`, when `start_mixing` notification is send.\nFrom this point, the mixer will produce an audio until `:schedule_eos` notification and `:end_of_stream` are received on all input pads.\n\nMixer mixes only raw audio (PCM), so some parser may be needed to precede it in pipeline.","ref":"Membrane.LiveAudioMixer.html"},{"type":"module","title":"Notifications - Membrane.LiveAudioMixer","doc":"- `:schedule_eos` -  mixer will send `end_of_stream` when it processes all input streams.\n  After sending `:schedule_eos` mixer will raise if it gets a new input pad.\n\n- {`:start_mixing`, latency} - mixer will start mixing audio after latency (non_neg_integer()).\n  Audio that will come before the notification will be buffered.\n\nInput pads can have offset - it tells how much timestamps differ from mixer time.","ref":"Membrane.LiveAudioMixer.html#module-notifications"},{"type":"module","title":"Element options - Membrane.LiveAudioMixer","doc":"Passed via struct `t:Membrane.LiveAudioMixer.t/0`\n\n- `prevent_clipping`  \n\n  ```\n  boolean()\n  ```\n  \n  Default value: `false`  \n  Defines how the mixer should act in the case when an overflow happens.\n  - If true, the wave will be scaled down, so a peak will become the maximal\n  value of the sample in the format. See `Membrane.AudioMixer.ClipPreventingAdder`.\n  - If false, overflow will be clipped to the maximal value of the sample in\n  the format. See `Membrane.AudioMixer.Adder`.\n\n\n- `native_mixer`  \n\n  ```\n  boolean()\n  ```\n  \n  Default value: `false`  \n  The value determines if mixer should use NIFs for mixing audio. Only\n  clip preventing version of native mixer is available.\n  See `Membrane.AudioMixer.NativeAdder`.\n\n\n\n- `latency`  \n\n  ```\n  non_neg_integer() | nil\n  ```\n  \n  Default value: `200 |> Membrane.Time.milliseconds()`  \n  The value determines after what time the clock will start interval that mixes audio in real time.\n  Latency is crucial to quality of output audio, the smaller the value, the more packets will be lost.\n  But the bigger the value, the bigger the latency of stream.\n  \n  Audio Mixer allows starting mixing earlier with parent_notification `:start_mixing`.\n  In this case, stream_format has to be passed through options.\n  \n  If notification `:start_mixing` is sent after mixing has started, the message will be discarded\n  \n  Start mixing manually:\n    * set latency to nil\n    * mixing has to be started manually by sending`:start_mixing` notification.\n\n\n\n- `stream_format`  \n\n  ```\n  RawAudio.t() | nil\n  ```\n  \n  Default value: `nil`  \n  The value defines a raw audio format of pads connected to the\n  element. It should be the same for all the pads.\n  It is necessary if `latency` is set to nil.","ref":"Membrane.LiveAudioMixer.html#module-element-options"},{"type":"module","title":"Pads - Membrane.LiveAudioMixer","doc":"### `:input`\n\nAccepted formats:\n```\n%RawAudio{sample_format: sample_format}\nwhen sample_format in [:s8, :s16le, :s16be, :s24le, :s24be, :s32le, :s32be]\n```\n\nDirection: | `:input`\nAvailability: | `:on_request`\nFlow control: | `:auto`\nPad options:\n\n- `offset`  \n\n  ```\n  Time.non_neg()\n  ```\n  \n  Default value: `0`  \n  Offset of the input audio at the pad.\n\n\n\n### `:output`\n\nAccepted formats:\n```\nRawAudio\n```\n\nDirection: | `:output`\nAvailability: | `:always`\nFlow control: | `:auto`","ref":"Membrane.LiveAudioMixer.html#module-pads"},{"type":"function","title":"Membrane.LiveAudioMixer.options/0","doc":"Returns description of options available for this module","ref":"Membrane.LiveAudioMixer.html#options/0"},{"type":"type","title":"Membrane.LiveAudioMixer.input_pad_opts/0","doc":"Options for pad `:input`","ref":"Membrane.LiveAudioMixer.html#t:input_pad_opts/0"},{"type":"type","title":"Membrane.LiveAudioMixer.t/0","doc":"Struct containing options for `Membrane.LiveAudioMixer`","ref":"Membrane.LiveAudioMixer.html#t:t/0"},{"type":"module","title":"Membrane.AudioMixer.Adder","doc":"Module responsible for mixing audio tracks (all in the same format, with the same number of\nchannels and sample rate). The result is a single track in the format mixed tracks are encoded in.\nIf overflow happens during mixing, it is being clipped to the max value of sample in this format.","ref":"Membrane.AudioMixer.Adder.html"},{"type":"type","title":"Membrane.AudioMixer.Adder.t/0","doc":"","ref":"Membrane.AudioMixer.Adder.html#t:t/0"},{"type":"module","title":"Membrane.AudioMixer.ClipPreventingAdder","doc":"Module responsible for mixing audio tracks (all in the same format, with the same number of\nchannels and sample rate). The result is a single track in the format mixed tracks are encoded in.\nIf overflow happens during mixing, a wave will be scaled down to the max sample value.\n\nDescription of the algorithm:\n  - Start with an empty queue\n  - Enqueue merged values while the sign of the values remains the same\n  - If the sign of values changes or adder is flushed:\n    - If none of the values overflows limits of the format, convert the queued values\n      to binary samples and return them\n    - Otherwise, scale down the queued values, so the peak of the wave will become\n      maximal (minimal) allowed value, then convert it to binary samples and return\n      them.","ref":"Membrane.AudioMixer.ClipPreventingAdder.html"},{"type":"type","title":"Membrane.AudioMixer.ClipPreventingAdder.t/0","doc":"","ref":"Membrane.AudioMixer.ClipPreventingAdder.html#t:t/0"},{"type":"module","title":"Membrane.AudioMixer.NativeAdder","doc":"Module responsible for mixing audio tracks (all in the same format, with the same number of\nchannels and sample rate). The result is a single track in the format mixed tracks are encoded in.\nIf overflow happens during mixing, a wave will be scaled down to the max sample value. Uses NIFs\nfor mixing.\n\nDescription of the algorithm:\n- Start with an empty queue\n- Enqueue merged values while the sign of the values remains the same\n- If the sign of values changes or adder is flushed:\n  - If none of the values overflows limits of the format, convert the queued values\n    to binary samples and return them\n  - Otherwise, scale down the queued values, so the peak of the wave will become\n    maximal (minimal) allowed value, then convert it to binary samples and return\n    them.","ref":"Membrane.AudioMixer.NativeAdder.html"},{"type":"module","title":"Membrane.LiveAudioMixer.LiveQueue","doc":"There are a lot of problems that the mixer can encounter while processing live audio streams:\n* packet loss resulting in small stream discontinuity\n* connection issues resulting in  complete lack of data\n* the need for enforcing max latency on the stream - packets that come too late have to be dropped\n\nThe LiveQueue tackles all those problems.\nIt has an independent queue for each stream.\nEvery gap caused by late or dropped packets are filled with silence.\nIf there is a need for more audio than there is in a queue, the missing part will also be filled with silence.","ref":"Membrane.LiveAudioMixer.LiveQueue.html"},{"type":"function","title":"Membrane.LiveAudioMixer.LiveQueue.add_buffer/3","doc":"Adds to a specific queue.\n\nWhen a buffer is too old it will be dropped\nWhen a part of a buffer is too old, only the part that is \"fresh\" will be added.\nWhen a whole buffer is \"fresh\", the whole buffer will be added.\nAll the wholes between audio packets will be filled with silence.\n\nThe state of the buffer, whether it's too old or not, is based on LiveQueue's `current_time`.","ref":"Membrane.LiveAudioMixer.LiveQueue.html#add_buffer/3"},{"type":"function","title":"Membrane.LiveAudioMixer.LiveQueue.add_queue/3","doc":"","ref":"Membrane.LiveAudioMixer.LiveQueue.html#add_queue/3"},{"type":"function","title":"Membrane.LiveAudioMixer.LiveQueue.all_queues_empty?/1","doc":"","ref":"Membrane.LiveAudioMixer.LiveQueue.html#all_queues_empty?/1"},{"type":"function","title":"Membrane.LiveAudioMixer.LiveQueue.get_audio/2","doc":"","ref":"Membrane.LiveAudioMixer.LiveQueue.html#get_audio/2"},{"type":"function","title":"Membrane.LiveAudioMixer.LiveQueue.init/1","doc":"","ref":"Membrane.LiveAudioMixer.LiveQueue.html#init/1"},{"type":"function","title":"Membrane.LiveAudioMixer.LiveQueue.remove_queue/2","doc":"Removes queue from a live queue.\n\nIf the queue is empty, it will be removed right away.\nOtherwise, it will be marked as `draining` and will be removed when it will get empty.","ref":"Membrane.LiveAudioMixer.LiveQueue.html#remove_queue/2"},{"type":"opaque","title":"Membrane.LiveAudioMixer.LiveQueue.t/0","doc":"","ref":"Membrane.LiveAudioMixer.LiveQueue.html#t:t/0"},{"type":"module","title":"Membrane.LiveAudioMixer.LiveQueue.Queue","doc":"The `Queue` module is responsible for storing a single live audio stream.","ref":"Membrane.LiveAudioMixer.LiveQueue.Queue.html"},{"type":"type","title":"Membrane.LiveAudioMixer.LiveQueue.Queue.t/0","doc":"","ref":"Membrane.LiveAudioMixer.LiveQueue.Queue.html#t:t/0"},{"type":"module","title":"Membrane.AudioInterleaver","doc":"Element responsible for interleaving several mono audio streams into single interleaved stream.\nAll input streams should be in the same raw audio format, defined by `input_stream_format` option.\n\nChannels are interleaved in order given in `order` option - currently required, no default available.\n\nEach input pad should be identified with your custom id (using `via_in(Pad.ref(:input, your_example_id)` )","ref":"Membrane.AudioInterleaver.html"},{"type":"module","title":"Element options - Membrane.AudioInterleaver","doc":"Passed via struct `t:Membrane.AudioInterleaver.t/0`\n\n- `input_stream_format`  \n\n  ```\n  RawAudio.t()\n  ```\n  \n  Default value: `nil`  \n  The value defines a raw audio format of pads connected to the\n  element. It should be the same for all the pads.\n\n\n- `frames_per_buffer`  \n\n  ```\n  pos_integer()\n  ```\n  \n  Default value: `2048`  \n  Assumed number of raw audio frames in each buffer.\n  Used when converting demand from buffers into bytes.\n\n\n\n- `order`  \n\n  ```\n  [any()]\n  ```\n  \n  ***Required***  \n  Order in which channels should be interleaved","ref":"Membrane.AudioInterleaver.html#module-element-options"},{"type":"module","title":"Pads - Membrane.AudioInterleaver","doc":"### `:input`\n\nAccepted formats:\n```\n%RawAudio{channels: 1}\n```\n```\nMembrane.RemoteStream\n```\n\nDirection: | `:input`\nAvailability: | `:on_request`\nFlow control: | `:manual`\nDemand unit: | `:bytes`\nPad options:\n\n- `offset`  \n\n  ```\n  Time.t()\n  ```\n  \n  Default value: `0`  \n  Offset of the input audio at the pad.\n\n\n\n### `:output`\n\nAccepted formats:\n```\nRawAudio\n```\n\nDirection: | `:output`\nAvailability: | `:always`\nFlow control: | `:manual`\nDemand unit: | `nil`","ref":"Membrane.AudioInterleaver.html#module-pads"},{"type":"function","title":"Membrane.AudioInterleaver.options/0","doc":"Returns description of options available for this module","ref":"Membrane.AudioInterleaver.html#options/0"},{"type":"type","title":"Membrane.AudioInterleaver.input_pad_opts/0","doc":"Options for pad `:input`","ref":"Membrane.AudioInterleaver.html#t:input_pad_opts/0"},{"type":"type","title":"Membrane.AudioInterleaver.t/0","doc":"Struct containing options for `Membrane.AudioInterleaver`","ref":"Membrane.AudioInterleaver.html#t:t/0"},{"type":"module","title":"Membrane.AudioInterleaver.DoInterleave","doc":"Module responsible for interleaving audio tracks (all in the same format, with 1\nchannel) in a given order.","ref":"Membrane.AudioInterleaver.DoInterleave.html"},{"type":"function","title":"Membrane.AudioInterleaver.DoInterleave.interleave/4","doc":"Order queues according to `order`, take `bytes_per_channel` from each queue\n(all queues must be at least `bytes_per_channel` long),\nand interleave them.","ref":"Membrane.AudioInterleaver.DoInterleave.html#interleave/4"},{"type":"extras","title":"Membrane Audio Mix Plugin","doc":"# Membrane Audio Mix Plugin\n\n[![Hex.pm](https://img.shields.io/hexpm/v/membrane_audio_mix_plugin.svg)](https://hex.pm/packages/membrane_audio_mix_plugin)\n[![API Docs](https://img.shields.io/badge/api-docs-yellow.svg?style=flat)](https://hexdocs.pm/membrane_audio_mix_plugin/)\n[![CircleCI](https://circleci.com/gh/membraneframework/membrane_audio_mix_plugin.svg?style=svg)](https://circleci.com/gh/membraneframework/membrane_audio_mix_plugin)\n\nPlugin providing elements for mixing and interleaving raw audio frames.\n\nIt is a part of [Membrane Multimedia Framework](https://membraneframework.org).","ref":"readme.html"},{"type":"extras","title":"Installation - Membrane Audio Mix Plugin","doc":"First, you need to install [MAD](https://www.underbit.com/products/mad/) on your system.","ref":"readme.html#installation"},{"type":"extras","title":"macOS - Membrane Audio Mix Plugin","doc":"```bash\nbrew install mad\n```","ref":"readme.html#macos"},{"type":"extras","title":"Ubuntu - Membrane Audio Mix Plugin","doc":"```bash\nsudo apt-get install libmad0-dev\n```\n\nThen, add the following line to your `deps` in `mix.exs`:\n\n```elixir\n\t{:membrane_audio_mix_plugin, \"~> 0.16.2\"}\n```\n\nand run `mix deps.get`.","ref":"readme.html#ubuntu"},{"type":"extras","title":"Description - Membrane Audio Mix Plugin","doc":"Elements operate only on raw audio (PCM), so some parser may be needed to precede them in a pipeline.\n\nThe elements expect to receive the same audio stream format on each input pad.\n\nAudio Mixers' input pads can have offset - in the case of a regular mixer it tells how much silence should be added before the first sample.\nThe live mixer will rewrite timestamps based on the value. Offset has to be positive.\n\nMixing and interleaving are tested only for integer audio formats.","ref":"readme.html#description"},{"type":"extras","title":"Mixer - Membrane Audio Mix Plugin","doc":"The Mixer adds samples from all pads. It has two strategies to deal with the overflow:\nscaling down waves and clipping.\n\nStream format can be additionally enforced by setting an element option (`:stream_format`)","ref":"readme.html#mixer"},{"type":"extras","title":"Interleaver - Membrane Audio Mix Plugin","doc":"This element joins several mono audio streams (with one channel) into one stream with interleaved channels.\n\nIf audio streams have different durations, all shorter streams are appended with silence to match the longest stream.\n\nEach channel must be named by providing an input pad name and the channel layout using those names must be provided (see [usage example](#audiointerleaver)).\n\nAll inputs have to be added before starting the pipeline and should not be changed during interleaver's work.\n\nStream format can be additionally enforced by setting an element option (`:stream_format`)","ref":"readme.html#interleaver"},{"type":"extras","title":"Live Mixer - Membrane Audio Mix Plugin","doc":"The Live Mixer adds samples from all pads. It has two strategies to deal with the overflow:\nscaling down waves and clipping. The Live Mixer will produce audio in real time meaning every interval there will be sent a buffer of fixed duration.\n\nIn situations where an audio packet is late or doesn't come, the mixer will fill the gaps with silence","ref":"readme.html#live-mixer"},{"type":"extras","title":"Usage Example - Membrane Audio Mix Plugin","doc":"","ref":"readme.html#usage-example"},{"type":"extras","title":"AudioMixer - Membrane Audio Mix Plugin","doc":"```elixir\ndefmodule Mixing.Pipeline do\n  use Membrane.Pipeline\n\n  @impl true\n  def handle_init(_ctx, _options) do\n    structure = [\n      child({:file_src, 1}, %Membrane.File.Source{location: \"/tmp/input_1.raw\"})\n      |> get_child(:mixer),\n\n      child({:file_src, 2}, %Membrane.File.Source{location: \"/tmp/input_2.raw\"})\n      |> via_in(:input, options: [offset: Membrane.Time.milliseconds(5000)])\n      |> get_child(:mixer),\n\n      child(:mixer, %Membrane.AudioMixer{\n        stream_format: %Membrane.RawAudio{\n          channels: 1,\n          sample_rate: 16_000,\n          sample_format: :s16le\n        }\n      })\n      |> child(:converter, %Membrane.FFmpeg.SWResample.Converter{\n        input_stream_format: %Membrane.RawAudio{channels: 1, sample_rate: 16_000, sample_format: :s16le},\n        output_stream_format: %Membrane.RawAudio{channels: 2, sample_rate: 48_000, sample_format: :s16le}\n      })\n      |> child(:player, Membrane.PortAudio.Sink)\n    ]\n\n    {[spec: structure], %{}}\n  end\nend\n```","ref":"readme.html#audiomixer"},{"type":"extras","title":"AudioInterleaver - Membrane Audio Mix Plugin","doc":"```elixir\ndefmodule Interleave.Pipeline do\n  use Membrane.Pipeline\n\n  alias Membrane.File.{Sink, Source}\n\n  @impl true\n  def handle_init(_ctx, {path_to_wav_1, path_to_wav_2}) do\n    structure = [\n      child({:file, 1}, %Source{location: path_to_wav_1})\n      |> child({:parser, 1}, Membrane.WAV.Parser)\n      |> get_child(:interleaver),\n\n      child({:file, 2}, %Source{location: path_to_wav_2})\n      |> child({:parser, 2}, Membrane.WAV.Parser)\n      |> get_child(:interleaver),\n\n\n      child(:interleaver, %Membrane.AudioInterleaver{\n        input_stream_format: %Membrane.RawAudio{\n          channels: 1,\n          sample_rate: 16_000,\n          sample_format: :s16le\n        },\n        order: [:left, :right]\n      })\n      |> child(:file_sink, %Sink{location: \"output.raw\"})\n    ]\n\n    {[spec: structure], %{}}\n  end\nend\n\n```","ref":"readme.html#audiointerleaver"},{"type":"extras","title":"Copyright and License - Membrane Audio Mix Plugin","doc":"Copyright 2021, [Software Mansion](https://swmansion.com/?utm_source=git&utm_medium=readme&utm_campaign=membrane)\n\n[![Software Mansion](https://logo.swmansion.com/logo?color=white&variant=desktop&width=200&tag=membrane-github)](https://swmansion.com/?utm_source=git&utm_medium=readme&utm_campaign=membrane)\n\nLicensed under the [Apache License, Version 2.0](LICENSE)","ref":"readme.html#copyright-and-license"},{"type":"extras","title":"LICENSE","doc":"Apache License\n                           Version 2.0, January 2004\n                        http://www.apache.org/licenses/\n\n   TERMS AND CONDITIONS FOR USE, REPRODUCTION, AND DISTRIBUTION\n\n   1. Definitions.\n\n      \"License\" shall mean the terms and conditions for use, reproduction,\n      and distribution as defined by Sections 1 through 9 of this document.\n\n      \"Licensor\" shall mean the copyright owner or entity authorized by\n      the copyright owner that is granting the License.\n\n      \"Legal Entity\" shall mean the union of the acting entity and all\n      other entities that control, are controlled by, or are under common\n      control with that entity. For the purposes of this definition,\n      \"control\" means (i) the power, direct or indirect, to cause the\n      direction or management of such entity, whether by contract or\n      otherwise, or (ii) ownership of fifty percent (50%) or more of the\n      outstanding shares, or (iii) beneficial ownership of such entity.\n\n      \"You\" (or \"Your\") shall mean an individual or Legal Entity\n      exercising permissions granted by this License.\n\n      \"Source\" form shall mean the preferred form for making modifications,\n      including but not limited to software source code, documentation\n      source, and configuration files.\n\n      \"Object\" form shall mean any form resulting from mechanical\n      transformation or translation of a Source form, including but\n      not limited to compiled object code, generated documentation,\n      and conversions to other media types.\n\n      \"Work\" shall mean the work of authorship, whether in Source or\n      Object form, made available under the License, as indicated by a\n      copyright notice that is included in or attached to the work\n      (an example is provided in the Appendix below).\n\n      \"Derivative Works\" shall mean any work, whether in Source or Object\n      form, that is based on (or derived from) the Work and for which the\n      editorial revisions, annotations, elaborations, or other modifications\n      represent, as a whole, an original work of authorship. For the purposes\n      of this License, Derivative Works shall not include works that remain\n      separable from, or merely link (or bind by name) to the interfaces of,\n      the Work and Derivative Works thereof.\n\n      \"Contribution\" shall mean any work of authorship, including\n      the original version of the Work and any modifications or additions\n      to that Work or Derivative Works thereof, that is intentionally\n      submitted to Licensor for inclusion in the Work by the copyright owner\n      or by an individual or Legal Entity authorized to submit on behalf of\n      the copyright owner. For the purposes of this definition, \"submitted\"\n      means any form of electronic, verbal, or written communication sent\n      to the Licensor or its representatives, including but not limited to\n      communication on electronic mailing lists, source code control systems,\n      and issue tracking systems that are managed by, or on behalf of, the\n      Licensor for the purpose of discussing and improving the Work, but\n      excluding communication that is conspicuously marked or otherwise\n      designated in writing by the copyright owner as \"Not a Contribution.\"\n\n      \"Contributor\" shall mean Licensor and any individual or Legal Entity\n      on behalf of whom a Contribution has been received by Licensor and\n      subsequently incorporated within the Work.\n\n   2. Grant of Copyright License. Subject to the terms and conditions of\n      this License, each Contributor hereby grants to You a perpetual,\n      worldwide, non-exclusive, no-charge, royalty-free, irrevocable\n      copyright license to reproduce, prepare Derivative Works of,\n      publicly display, publicly perform, sublicense, and distribute the\n      Work and such Derivative Works in Source or Object form.\n\n   3. Grant of Patent License. Subject to the terms and conditions of\n      this License, each Contributor hereby grants to You a perpetual,\n      worldwide, non-exclusive, no-charge, royalty-free, irrevocable\n      (except as stated in this section) patent license to make, have made,\n      use, offer to sell, sell, import, and otherwise transfer the Work,\n      where such license applies only to those patent claims licensable\n      by such Contributor that are necessarily infringed by their\n      Contribution(s) alone or by combination of their Contribution(s)\n      with the Work to which such Contribution(s) was submitted. If You\n      institute patent litigation against any entity (including a\n      cross-claim or counterclaim in a lawsuit) alleging that the Work\n      or a Contribution incorporated within the Work constitutes direct\n      or contributory patent infringement, then any patent licenses\n      granted to You under this License for that Work shall terminate\n      as of the date such litigation is filed.\n\n   4. Redistribution. You may reproduce and distribute copies of the\n      Work or Derivative Works thereof in any medium, with or without\n      modifications, and in Source or Object form, provided that You\n      meet the following conditions:\n\n      (a) You must give any other recipients of the Work or\n          Derivative Works a copy of this License; and\n\n      (b) You must cause any modified files to carry prominent notices\n          stating that You changed the files; and\n\n      (c) You must retain, in the Source form of any Derivative Works\n          that You distribute, all copyright, patent, trademark, and\n          attribution notices from the Source form of the Work,\n          excluding those notices that do not pertain to any part of\n          the Derivative Works; and\n\n      (d) If the Work includes a \"NOTICE\" text file as part of its\n          distribution, then any Derivative Works that You distribute must\n          include a readable copy of the attribution notices contained\n          within such NOTICE file, excluding those notices that do not\n          pertain to any part of the Derivative Works, in at least one\n          of the following places: within a NOTICE text file distributed\n          as part of the Derivative Works; within the Source form or\n          documentation, if provided along with the Derivative Works; or,\n          within a display generated by the Derivative Works, if and\n          wherever such third-party notices normally appear. The contents\n          of the NOTICE file are for informational purposes only and\n          do not modify the License. You may add Your own attribution\n          notices within Derivative Works that You distribute, alongside\n          or as an addendum to the NOTICE text from the Work, provided\n          that such additional attribution notices cannot be construed\n          as modifying the License.\n\n      You may add Your own copyright statement to Your modifications and\n      may provide additional or different license terms and conditions\n      for use, reproduction, or distribution of Your modifications, or\n      for any such Derivative Works as a whole, provided Your use,\n      reproduction, and distribution of the Work otherwise complies with\n      the conditions stated in this License.\n\n   5. Submission of Contributions. Unless You explicitly state otherwise,\n      any Contribution intentionally submitted for inclusion in the Work\n      by You to the Licensor shall be under the terms and conditions of\n      this License, without any additional terms or conditions.\n      Notwithstanding the above, nothing herein shall supersede or modify\n      the terms of any separate license agreement you may have executed\n      with Licensor regarding such Contributions.\n\n   6. Trademarks. This License does not grant permission to use the trade\n      names, trademarks, service marks, or product names of the Licensor,\n      except as required for reasonable and customary use in describing the\n      origin of the Work and reproducing the content of the NOTICE file.\n\n   7. Disclaimer of Warranty. Unless required by applicable law or\n      agreed to in writing, Licensor provides the Work (and each\n      Contributor provides its Contributions) on an \"AS IS\" BASIS,\n      WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or\n      implied, including, without limitation, any warranties or conditions\n      of TITLE, NON-INFRINGEMENT, MERCHANTABILITY, or FITNESS FOR A\n      PARTICULAR PURPOSE. You are solely responsible for determining the\n      appropriateness of using or redistributing the Work and assume any\n      risks associated with Your exercise of permissions under this License.\n\n   8. Limitation of Liability. In no event and under no legal theory,\n      whether in tort (including negligence), contract, or otherwise,\n      unless required by applicable law (such as deliberate and grossly\n      negligent acts) or agreed to in writing, shall any Contributor be\n      liable to You for damages, including any direct, indirect, special,\n      incidental, or consequential damages of any character arising as a\n      result of this License or out of the use or inability to use the\n      Work (including but not limited to damages for loss of goodwill,\n      work stoppage, computer failure or malfunction, or any and all\n      other commercial damages or losses), even if such Contributor\n      has been advised of the possibility of such damages.\n\n   9. Accepting Warranty or Additional Liability. While redistributing\n      the Work or Derivative Works thereof, You may choose to offer,\n      and charge a fee for, acceptance of support, warranty, indemnity,\n      or other liability obligations and/or rights consistent with this\n      License. However, in accepting such obligations, You may act only\n      on Your own behalf and on Your sole responsibility, not on behalf\n      of any other Contributor, and only if You agree to indemnify,\n      defend, and hold each Contributor harmless for any liability\n      incurred by, or claims asserted against, such Contributor by reason\n      of your accepting any such warranty or additional liability.\n\n   END OF TERMS AND CONDITIONS\n\n   APPENDIX: How to apply the Apache License to your work.\n\n      To apply the Apache License to your work, attach the following\n      boilerplate notice, with the fields enclosed by brackets \"[]\"\n      replaced with your own identifying information. (Don't include\n      the brackets!)  The text should be enclosed in the appropriate\n      comment syntax for the file format. We also recommend that a\n      file or class name and description of purpose be included on the\n      same \"printed page\" as the copyright notice for easier\n      identification within third-party archives.\n\n   Copyright 2020 Software Mansion\n\n   Licensed under the Apache License, Version 2.0 (the \"License\");\n   you may not use this file except in compliance with the License.\n   You may obtain a copy of the License at\n\n       http://www.apache.org/licenses/LICENSE-2.0\n\n   Unless required by applicable law or agreed to in writing, software\n   distributed under the License is distributed on an \"AS IS\" BASIS,\n   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.\n   See the License for the specific language governing permissions and\n   limitations under the License.","ref":"license.html"}],"content_type":"text/markdown","producer":{"name":"ex_doc","version":[48,46,51,52,46,50]}}